@@ -6,12 +6,14 @@ package=gstreamer
 srcfile=gst/gst.c
 
 # a quick cvs co to ease the transition
-if test ! -d common; then
-  if test -f CVS/Tag; then
-    # get everything from CVS/Tag from second character on
+if test ! -d common;
+then
+  echo "+ getting common/ from cvs"
+  if test -e CVS/Tag
+  then
     TAG="-r `tail -c +2 CVS/Tag`"
   fi
-  echo "+ getting common from cvs"; cvs co $TAG common
+  cvs co $TAG common
 fi
 
 # source helper functions
@@ -23,19 +25,19 @@ then
 fi
 . common/gst-autogen.sh
 
-CONFIGURE_DEF_OPT='--enable-maintainer-mode --enable-plugin-builddir --enable-failing-tests --enable-poisoning'
+CONFIGURE_DEF_OPT='--enable-maintainer-mode --enable-failing-tests --enable-poisoning --enable-gtk-doc --enable-docbook'
 
 autogen_options $@
 
 echo -n "+ check for build tools"
 if test ! -z "$NOCHECK"; then echo ": skipped version checks"; else  echo; fi
-version_check "autoconf" "$AUTOCONF autoconf autoconf-2.54 autoconf-2.53 autoconf-2.52" \
+version_check "autoconf" "$AUTOCONF autoconf autoconf259 autoconf257 autoconf-2.54 autoconf-2.53 autoconf253 autoconf-2.52 autoconf252" \
               "ftp://ftp.gnu.org/pub/gnu/autoconf/" 2 52 || DIE=1
-version_check "automake" "$AUTOMAKE automake automake-1.7 automake17 automake-1.6" \
+version_check "automake" "$AUTOMAKE automake automake-1.9 automake19 automake-1.8 automake18 automake-1.7 automake17 automake-1.6 automake16" \
               "ftp://ftp.gnu.org/pub/gnu/automake/" 1 6 || DIE=1
 version_check "autopoint" "autopoint" \
               "ftp://ftp.gnu.org/pub/gnu/gettext/" 0 11 5 || DIE=1
-version_check "libtoolize" "libtoolize" \
+version_check "libtoolize" "libtoolize libtoolize15" \
               "ftp://ftp.gnu.org/pub/gnu/libtool/" 1 5 0 || DIE=1
 version_check "pkg-config" "" \
               "http://www.freedesktop.org/software/pkgconfig" 0 8 0 || DIE=1
@@ -65,7 +67,7 @@ if test -x mkinstalldirs; then rm mkinstalldirs; fi
 #    first remove patch if necessary, then run autopoint, then reapply
 if test -f po/Makefile.in.in;
 then
-  patch -p0 -R < common/gettext.patch
+  patch -p0 -R --forward < common/gettext.patch
 fi
 tool_run "$autopoint" "--force" "patch -p0 < common/gettext.patch"
 patch -p0 < common/gettext.patch
