@@ -17,6 +17,52 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstcaps
+ * @short_description: Structure describing sets of media formats
+ * @see_also: #GstStructure
+ *
+ * Caps (capabilities) are lighweight refcounted objects describing media types.
+ * They are composed of an array of #GstStructure.
+ *
+ * Caps are exposed on #GstPadTemplate to describe all possible types a
+ * given pad can handle. They are also stored in the registry along with
+ * a description of the element.
+ *
+ * Caps are exposed on the element pads using the gst_pad_get_caps() pad 
+ * function. This function describes the possible types that the pad can 
+ * handle or produce at runtime.
+ *
+ * Caps are also attached to buffers to describe to content of the data
+ * pointed to by the buffer with gst_buffer_set_caps(). Caps attached to
+ * a #GstBuffer allow for format negotiation upstream and downstream.
+ *
+ * A #GstCaps can be constructed with the following code fragment:
+ *
+ * <example>
+ *  <title>Creating caps</title>
+ *  <programlisting>
+ *  GstCaps *caps;
+ *  caps = gst_caps_new_simple ("video/x-raw-yuv",
+ *       "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
+ *       "framerate", G_TYPE_DOUBLE, 25.0,
+ *       "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
+ *       "width", G_TYPE_INT, 320, 
+ *       "height", G_TYPE_INT, 240, 
+ *       NULL);
+ *  </programlisting>
+ * </example>
+ *
+ * A #GstCaps is fixed when it has no properties with ranges or lists. Use
+ * gst_caps_is_fixed() to test for fixed caps. Only fixed caps can be
+ * set on a #GstPad or #GstBuffer.
+ *
+ * Various methods exist to work with the media types such as subtracting
+ * or intersecting.
+ *
+ * Last reviewed on 2005-11-23 (0.9.5)
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -26,10 +72,12 @@
 #include "gst_private.h"
 #include <gst/gst.h>
 
+#define DEBUG_REFCOUNT
+
 #define CAPS_POISON(caps) G_STMT_START{ \
   if (caps) { \
     GstCaps *_newcaps = gst_caps_copy (caps); \
-    gst_caps_free(caps); \
+    gst_caps_unref(caps); \
     caps = _newcaps; \
   } \
 } G_STMT_END
@@ -40,13 +88,15 @@
     structure = _newstruct; \
   } \
 } G_STMT_END
+#define IS_WRITABLE(caps) \
+  (g_atomic_int_get (&(caps)->refcount) == 1)
 
 
 static void gst_caps_transform_to_string (const GValue * src_value,
     GValue * dest_value);
 static gboolean gst_caps_from_string_inplace (GstCaps * caps,
     const gchar * string);
-static GstCaps *gst_caps_copy_conditional (const GstCaps * src);
+static GstCaps *gst_caps_copy_conditional (GstCaps * src);
 
 GType
 gst_caps_get_type (void)
@@ -56,7 +106,7 @@ gst_caps_get_type (void)
   if (!gst_caps_type) {
     gst_caps_type = g_boxed_type_register_static ("GstCaps",
         (GBoxedCopyFunc) gst_caps_copy_conditional,
-        (GBoxedFreeFunc) gst_caps_free);
+        (GBoxedFreeFunc) gst_caps_unref);
 
     g_value_register_transform_func (gst_caps_type,
         G_TYPE_STRING, gst_caps_transform_to_string);
@@ -72,6 +122,7 @@ gst_caps_get_type (void)
  *
  * Creates a new #GstCaps that is empty.  That is, the returned
  * #GstCaps contains no media formats.
+ * Caller is responsible for unreffing the returned caps.
  *
  * Returns: the new #GstCaps
  */
@@ -80,9 +131,14 @@ gst_caps_new_empty (void)
 {
   GstCaps *caps = g_new0 (GstCaps, 1);
 
+  g_atomic_int_inc (&caps->refcount);
   caps->type = GST_TYPE_CAPS;
   caps->structs = g_ptr_array_new ();
 
+#ifdef DEBUG_REFCOUNT
+  GST_CAT_LOG (GST_CAT_CAPS, "created caps %p", caps);
+#endif
+
   return caps;
 }
 
@@ -112,7 +168,8 @@ gst_caps_new_any (void)
  *
  * Creates a new #GstCaps that contains one #GstStructure.  The
  * structure is defined by the arguments, which have the same format
- * as @gst_structure_new().
+ * as gst_structure_new().
+ * Caller is responsible for unreffing the returned caps.
  *
  * Returns: the new #GstCaps
  */
@@ -188,8 +245,14 @@ gst_caps_new_full_valist (GstStructure * structure, va_list var_args)
  * gst_caps_copy:
  * @caps: the #GstCaps to copy
  *
- * Deeply copies a #GstCaps, including all structures and all the
- * structures' values.
+ * Creates a new #GstCaps as a copy of the old @caps. The new caps will have a
+ * refcount of 1, owned by the caller. The structures are copied as well.
+ *
+ * Note that this function is the semantic equivalent of a gst_caps_ref()
+ * followed by a gst_caps_make_writable(). If you only want to hold on to a
+ * reference to the data, you should use gst_caps_ref().
+ *
+ * When you are finished with the caps, call gst_caps_unref() on it.
  *
  * Returns: the new #GstCaps
  */
@@ -198,7 +261,7 @@ gst_caps_copy (const GstCaps * caps)
 {
   GstCaps *newcaps;
   GstStructure *structure;
-  int i;
+  guint i;
 
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
 
@@ -213,23 +276,18 @@ gst_caps_copy (const GstCaps * caps)
   return newcaps;
 }
 
-/**
- * gst_caps_free:
- * @caps: the #GstCaps to free
- *
- * Frees a #GstCaps and all its structures and the structures'
- * values.
- */
-void
-gst_caps_free (GstCaps * caps)
+static void
+_gst_caps_free (GstCaps * caps)
 {
   GstStructure *structure;
-  int i;
+  guint i;
 
-  g_return_if_fail (GST_IS_CAPS (caps));
+  /* The refcount must be 0, but since we're only called by gst_caps_unref,
+   * don't bother testing. */
 
   for (i = 0; i < caps->structs->len; i++) {
-    structure = gst_caps_get_structure (caps, i);
+    structure = (GstStructure *) gst_caps_get_structure (caps, i);
+    gst_structure_set_parent_refcount (structure, NULL);
     gst_structure_free (structure);
   }
   g_ptr_array_free (caps->structs, TRUE);
@@ -239,22 +297,135 @@ gst_caps_free (GstCaps * caps)
   g_free (caps);
 }
 
+/**
+ * gst_caps_make_writable:
+ * @caps: the #GstCaps to make writable
+ *
+ * Returns a writable copy of @caps.
+ *
+ * If there is only one reference count on @caps, the caller must be the owner,
+ * and so this function will return the caps object unchanged. If on the other
+ * hand there is more than one reference on the object, a new caps object will
+ * be returned. The caller's reference on @caps will be removed, and instead the
+ * caller will own a reference to the returned object.
+ *
+ * In short, this function unrefs the caps in the argument and refs the caps
+ * that it returns. Don't access the argument after calling this function. See
+ * also: gst_caps_ref().
+ *
+ * Returns: the same #GstCaps object.
+ */
+GstCaps *
+gst_caps_make_writable (GstCaps * caps)
+{
+  GstCaps *copy;
+
+  g_return_val_if_fail (caps != NULL, NULL);
+
+  /* we are the only instance reffing this caps */
+  if (g_atomic_int_get (&caps->refcount) == 1)
+    return caps;
+
+  /* else copy */
+  copy = gst_caps_copy (caps);
+  gst_caps_unref (caps);
+
+  return copy;
+}
+
+/**
+ * gst_caps_ref:
+ * @caps: the #GstCaps to reference
+ *
+ * Add a reference to a #GstCaps object.
+ *
+ * From this point on, until the caller calls gst_caps_unref() or
+ * gst_caps_make_writable(), it is guaranteed that the caps object will not
+ * change. This means its structures won't change, etc. To use a #GstCaps
+ * object, you must always have a refcount on it -- either the one made
+ * implicitly by gst_caps_new(), or via taking one explicitly with this
+ * function.
+ *
+ * Returns: the same #GstCaps object.
+ */
+GstCaps *
+gst_caps_ref (GstCaps * caps)
+{
+  g_return_val_if_fail (caps != NULL, NULL);
+
+#ifdef DEBUG_REFCOUNT
+  GST_CAT_LOG (GST_CAT_CAPS, "%p %d->%d", caps,
+      GST_CAPS_REFCOUNT_VALUE (caps), GST_CAPS_REFCOUNT_VALUE (caps) + 1);
+#endif
+  g_return_val_if_fail (GST_CAPS_REFCOUNT_VALUE (caps) > 0, NULL);
+
+  g_atomic_int_inc (&caps->refcount);
+
+  return caps;
+}
+
+/**
+ * gst_caps_unref:
+ * @caps: the #GstCaps to unref
+ *
+ * Unref a #GstCaps and and free all its structures and the
+ * structures' values when the refcount reaches 0.
+ */
+void
+gst_caps_unref (GstCaps * caps)
+{
+  g_return_if_fail (caps != NULL);
+
+#ifdef DEBUG_REFCOUNT
+  GST_CAT_LOG (GST_CAT_CAPS, "%p %d->%d", caps,
+      GST_CAPS_REFCOUNT_VALUE (caps), GST_CAPS_REFCOUNT_VALUE (caps) - 1);
+#endif
+
+  g_return_if_fail (GST_CAPS_REFCOUNT_VALUE (caps) > 0);
+
+  /* if we ended up with the refcount at zero, free the caps */
+  if (g_atomic_int_dec_and_test (&caps->refcount)) {
+    _gst_caps_free (caps);
+  }
+}
+
+GType
+gst_static_caps_get_type (void)
+{
+  static GType staticcaps_type = 0;
+
+  if (!staticcaps_type) {
+    staticcaps_type = g_pointer_type_register_static ("GstStaticCaps");
+  }
+  return staticcaps_type;
+}
+
+
 /**
  * gst_static_caps_get:
  * @static_caps: the #GstStaticCaps to convert
  *
  * Converts a #GstStaticCaps to a #GstCaps.
  *
- * Returns: the new #GstCaps
+ * Returns: A pointer to the #GstCaps. Unref after usage. Since the
+ * core holds an additional ref to the returned caps,
+ * use gst_caps_make_writable() on the returned caps to modify it.
  */
-const GstCaps *
+GstCaps *
 gst_static_caps_get (GstStaticCaps * static_caps)
 {
   GstCaps *caps = (GstCaps *) static_caps;
   gboolean ret;
 
   if (caps->type == 0) {
+    if (static_caps->string == NULL) {
+      g_warning ("static caps is NULL");
+      return NULL;
+    }
+
     caps->type = GST_TYPE_CAPS;
+    /* initialize the caps to a refcount of 1 so the caps can be writable... */
+    gst_atomic_int_set (&caps->refcount, 1);
     caps->structs = g_ptr_array_new ();
     ret = gst_caps_from_string_inplace (caps, static_caps->string);
 
@@ -262,20 +433,32 @@ gst_static_caps_get (GstStaticCaps * static_caps)
       g_critical ("Could not convert static caps \"%s\"", static_caps->string);
     }
   }
+  /* ref the caps, makes it not writable */
+  gst_caps_ref (caps);
 
   return caps;
 }
 
 /* manipulation */
 
+static GstStructure *
+gst_caps_remove_and_get_structure (GstCaps * caps, guint idx)
+{
+  /* don't use index_fast, gst_caps_simplify relies on the order */
+  GstStructure *s = g_ptr_array_remove_index (caps->structs, idx);
+
+  gst_structure_set_parent_refcount (s, NULL);
+  return s;
+}
+
 /**
  * gst_caps_append:
  * @caps1: the #GstCaps that will be appended to
  * @caps2: the #GstCaps to append
  *
- * Appends the structures contained in @caps2 to @caps1.  The structures
- * in @caps2 are not copied -- they are transferred to @caps1, and then
- * @caps2 is freed.
+ * Appends the structures contained in @caps2 to @caps1. The structures in
+ * @caps2 are not copied -- they are transferred to @caps1, and then @caps2 is
+ * freed. If either caps is ANY, the resulting caps will be ANY.
  */
 void
 gst_caps_append (GstCaps * caps1, GstCaps * caps2)
@@ -285,6 +468,8 @@ gst_caps_append (GstCaps * caps1, GstCaps * caps2)
 
   g_return_if_fail (GST_IS_CAPS (caps1));
   g_return_if_fail (GST_IS_CAPS (caps2));
+  g_return_if_fail (IS_WRITABLE (caps1));
+  g_return_if_fail (IS_WRITABLE (caps2));
 
 #ifdef USE_POISONING
   CAPS_POISON (caps2);
@@ -292,21 +477,19 @@ gst_caps_append (GstCaps * caps1, GstCaps * caps2)
   if (gst_caps_is_any (caps1) || gst_caps_is_any (caps2)) {
     /* FIXME: this leaks */
     caps1->flags |= GST_CAPS_FLAGS_ANY;
-    for (i = 0; i < caps2->structs->len; i++) {
-      structure = gst_caps_get_structure (caps2, i);
-      gst_structure_remove_all_fields (structure);
+    for (i = caps2->structs->len - 1; i >= 0; i--) {
+      structure = gst_caps_remove_and_get_structure (caps2, i);
+      gst_structure_free (structure);
     }
   } else {
-    for (i = 0; i < caps2->structs->len; i++) {
-      structure = gst_caps_get_structure (caps2, i);
+    int len = caps2->structs->len;
+
+    for (i = 0; i < len; i++) {
+      structure = gst_caps_remove_and_get_structure (caps2, 0);
       gst_caps_append_structure (caps1, structure);
     }
   }
-  g_ptr_array_free (caps2->structs, TRUE);
-#ifdef USE_POISONING
-  memset (caps2, 0xff, sizeof (GstCaps));
-#endif
-  g_free (caps2);
+  gst_caps_unref (caps2);       /* guaranteed to free it */
 }
 
 /**
@@ -321,30 +504,26 @@ void
 gst_caps_append_structure (GstCaps * caps, GstStructure * structure)
 {
   g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
 
   if (structure) {
+    g_return_if_fail (structure->parent_refcount == NULL);
 #if 0
 #ifdef USE_POISONING
     STRUCTURE_POISON (structure);
 #endif
 #endif
+    gst_structure_set_parent_refcount (structure, &caps->refcount);
     g_ptr_array_add (caps->structs, structure);
   }
 }
 
-static GstStructure *
-gst_caps_remove_and_get_structure (GstCaps * caps, guint idx)
-{
-  /* don't use index_fast, gst_caps_simplify relies on the order */
-  return g_ptr_array_remove_index (caps->structs, idx);
-}
-
 /*
  * gst_caps_remove_structure:
  * @caps: the #GstCaps to remove from
  * @idx: Index of the structure to remove
  *
- * removes the stucture with the given index from the list of structures 
+ * removes the stucture with the given index from the list of structures
  * contained in @caps.
  */
 void
@@ -354,6 +533,7 @@ gst_caps_remove_structure (GstCaps * caps, guint idx)
 
   g_return_if_fail (caps != NULL);
   g_return_if_fail (idx <= gst_caps_get_size (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
 
   structure = gst_caps_remove_and_get_structure (caps, idx);
   gst_structure_free (structure);
@@ -361,11 +541,11 @@ gst_caps_remove_structure (GstCaps * caps, guint idx)
 
 /**
  * gst_caps_split_one:
- * @caps: 
+ * @caps: a #GstCaps
  *
  * This function is not implemented.
  *
- * Returns:
+ * Returns: NULL
  */
 GstCaps *
 gst_caps_split_one (GstCaps * caps)
@@ -384,7 +564,7 @@ gst_caps_split_one (GstCaps * caps)
  *
  * Returns: the number of structures that @caps contains
  */
-int
+guint
 gst_caps_get_size (const GstCaps * caps)
 {
   g_return_val_if_fail (GST_IS_CAPS (caps), 0);
@@ -397,18 +577,18 @@ gst_caps_get_size (const GstCaps * caps)
  * @caps: a #GstCaps
  * @index: the index of the structure
  *
- * Finds the structure in @caps that has the index @index, and 
+ * Finds the structure in @caps that has the index @index, and
  * returns it.
  *
  * WARNING: This function takes a const GstCaps *, but returns a
  * non-const GstStructure *.  This is for programming convenience --
  * the caller should be aware that structures inside a constant
- * @GstCaps should not be modified.
+ * #GstCaps should not be modified.
  *
  * Returns: a pointer to the #GstStructure corresponding to @index
  */
 GstStructure *
-gst_caps_get_structure (const GstCaps * caps, int index)
+gst_caps_get_structure (const GstCaps * caps, guint index)
 {
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
   g_return_val_if_fail (index >= 0, NULL);
@@ -418,16 +598,17 @@ gst_caps_get_structure (const GstCaps * caps, int index)
 }
 
 /**
- * gst_caps_copy_1:
- * @caps: the @GstCaps to copy
+ * gst_caps_copy_nth:
+ * @caps: the #GstCaps to copy
+ * @nth: the nth structure to copy
  *
- * Creates a new @GstCaps and appends a copy of the first structure
+ * Creates a new #GstCaps and appends a copy of the nth structure
  * contained in @caps.
  *
- * Returns: the new @GstCaps
+ * Returns: the new #GstCaps
  */
 GstCaps *
-gst_caps_copy_1 (const GstCaps * caps)
+gst_caps_copy_nth (const GstCaps * caps, guint nth)
 {
   GstCaps *newcaps;
   GstStructure *structure;
@@ -437,23 +618,44 @@ gst_caps_copy_1 (const GstCaps * caps)
   newcaps = gst_caps_new_empty ();
   newcaps->flags = caps->flags;
 
-  if (caps->structs->len > 0) {
-    structure = gst_caps_get_structure (caps, 0);
+  if (caps->structs->len > nth) {
+    structure = gst_caps_get_structure (caps, nth);
     gst_caps_append_structure (newcaps, gst_structure_copy (structure));
   }
 
   return newcaps;
 }
 
+/**
+ * gst_caps_truncate:
+ * @caps: the #GstCaps to truncate
+ *
+ * Destructively discard all but the first structure from @caps. Useful when
+ * fixating. @caps must be writable.
+ */
+void
+gst_caps_truncate (GstCaps * caps)
+{
+  gint i;
+
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
+
+  i = caps->structs->len - 1;
+
+  while (i > 0)
+    gst_caps_remove_structure (caps, i--);
+}
+
 /**
  * gst_caps_set_simple:
- * @caps: the @GstCaps to set
+ * @caps: the #GstCaps to set
  * @field: first field to set
  * @...: additional parameters
  *
  * Sets fields in a simple #GstCaps.  A simple #GstCaps is one that
  * only has one structure.  The arguments must be passed in the same
- * manner as @gst_structure_set(), and be NULL-terminated.
+ * manner as gst_structure_set(), and be NULL-terminated.
  */
 void
 gst_caps_set_simple (GstCaps * caps, char *field, ...)
@@ -463,6 +665,7 @@ gst_caps_set_simple (GstCaps * caps, char *field, ...)
 
   g_return_if_fail (GST_IS_CAPS (caps));
   g_return_if_fail (caps->structs->len == 1);
+  g_return_if_fail (IS_WRITABLE (caps));
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -473,13 +676,13 @@ gst_caps_set_simple (GstCaps * caps, char *field, ...)
 
 /**
  * gst_caps_set_simple_valist:
- * @caps: the @GstCaps to copy
+ * @caps: the #GstCaps to copy
  * @field: first field to set
  * @varargs: additional parameters
  *
  * Sets fields in a simple #GstCaps.  A simple #GstCaps is one that
  * only has one structure.  The arguments must be passed in the same
- * manner as @gst_structure_set(), and be NULL-terminated.
+ * manner as gst_structure_set(), and be NULL-terminated.
  */
 void
 gst_caps_set_simple_valist (GstCaps * caps, char *field, va_list varargs)
@@ -488,6 +691,7 @@ gst_caps_set_simple_valist (GstCaps * caps, char *field, va_list varargs)
 
   g_return_if_fail (GST_IS_CAPS (caps));
   g_return_if_fail (caps->structs->len != 1);
+  g_return_if_fail (IS_WRITABLE (caps));
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -498,7 +702,7 @@ gst_caps_set_simple_valist (GstCaps * caps, char *field, va_list varargs)
 
 /**
  * gst_caps_is_any:
- * @caps: the @GstCaps to test
+ * @caps: the #GstCaps to test
  *
  * Determines if @caps represents any media format.
  *
@@ -514,7 +718,7 @@ gst_caps_is_any (const GstCaps * caps)
 
 /**
  * gst_caps_is_empty:
- * @caps: the @GstCaps to test
+ * @caps: the #GstCaps to test
  *
  * Determines if @caps represents no media formats.
  *
@@ -531,38 +735,18 @@ gst_caps_is_empty (const GstCaps * caps)
   return (caps->structs == NULL) || (caps->structs->len == 0);
 }
 
-/**
- * gst_caps_is_chained:
- * @caps: the @GstCaps to test
- *
- * Determines if @caps contains multiple #GstStructures.
- *
- * This function is deprecated, and should not be used in new code.
- * Use #gst_caps_is_simple() instead.
- *
- * Returns: TRUE if @caps contains more than one structure
- */
-gboolean
-gst_caps_is_chained (const GstCaps * caps)
-{
-  g_return_val_if_fail (GST_IS_CAPS (caps), FALSE);
-
-  return (caps->structs->len > 1);
-}
-
 static gboolean
-gst_caps_is_fixed_foreach (GQuark field_id, GValue * value, gpointer unused)
+gst_caps_is_fixed_foreach (GQuark field_id, const GValue * value,
+    gpointer unused)
 {
-  GType type = G_VALUE_TYPE (value);
-
-  return gst_type_is_fixed (type);
+  return gst_value_is_fixed (value);
 }
 
 /**
  * gst_caps_is_fixed:
- * @caps: the @GstCaps to test
+ * @caps: the #GstCaps to test
  *
- * Fixed @GstCaps describe exactly one format, that is, they have exactly
+ * Fixed #GstCaps describe exactly one format, that is, they have exactly
  * one structure, and each field in the structure describes a fixed type.
  * Examples of non-fixed types are GST_TYPE_INT_RANGE and GST_TYPE_LIST.
  *
@@ -584,7 +768,8 @@ gst_caps_is_fixed (const GstCaps * caps)
 }
 
 static gboolean
-gst_structure_is_equal_foreach (GQuark field_id, GValue * val2, gpointer data)
+gst_structure_is_equal_foreach (GQuark field_id, const GValue * val2,
+    gpointer data)
 {
   GstStructure *struct1 = (GstStructure *) data;
   const GValue *val1 = gst_structure_id_get_value (struct1, field_id);
@@ -677,7 +862,7 @@ gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
 
   caps = gst_caps_subtract (subset, superset);
   ret = gst_caps_is_empty (caps);
-  gst_caps_free (caps);
+  gst_caps_unref (caps);
   return ret;
 }
 
@@ -695,8 +880,14 @@ gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
 gboolean
 gst_caps_is_equal (const GstCaps * caps1, const GstCaps * caps2)
 {
-  g_return_val_if_fail (caps1 != NULL, FALSE);
-  g_return_val_if_fail (caps2 != NULL, FALSE);
+  /* NULL <-> NULL is allowed here */
+  if (caps1 == caps2)
+    return TRUE;
+
+  /* one of them NULL => they are different (can't be both NULL because
+   * we checked that above) */
+  if (caps1 == NULL || caps2 == NULL)
+    return FALSE;
 
   if (gst_caps_is_fixed (caps1) && gst_caps_is_fixed (caps2))
     return gst_caps_is_equal_fixed (caps1, caps2);
@@ -713,7 +904,8 @@ typedef struct
 IntersectData;
 
 static gboolean
-gst_caps_structure_intersect_field (GQuark id, GValue * val1, gpointer data)
+gst_caps_structure_intersect_field (GQuark id, const GValue * val1,
+    gpointer data)
 {
   IntersectData *idata = (IntersectData *) data;
   GValue dest_value = { 0 };
@@ -820,10 +1012,13 @@ gst_caps_structure_union (const GstStructure * struct1,
 GstCaps *
 gst_caps_intersect (const GstCaps * caps1, const GstCaps * caps2)
 {
-  int i, j;
+  guint64 i;                    /* index can be up to 2 * G_MAX_UINT */
+  guint j, k;
+
   GstStructure *struct1;
   GstStructure *struct2;
   GstCaps *dest;
+  GstStructure *istruct;
 
   g_return_val_if_fail (GST_IS_CAPS (caps1), NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps2), NULL);
@@ -837,19 +1032,46 @@ gst_caps_intersect (const GstCaps * caps1, const GstCaps * caps2)
     return gst_caps_copy (caps1);
 
   dest = gst_caps_new_empty ();
-  for (i = 0; i < caps1->structs->len; i++) {
-    struct1 = gst_caps_get_structure (caps1, i);
-    for (j = 0; j < caps2->structs->len; j++) {
-      GstStructure *istruct;
 
-      struct2 = gst_caps_get_structure (caps2, j);
+  /* run zigzag on top line then right line, this preserves the caps order
+   * much better than a simple loop.
+   *
+   * This algorithm zigzags over the caps structures as demonstrated in
+   * the folowing matrix:
+   *
+   *          caps1
+   *       +-------------
+   *       | 1  2  4  7
+   * caps2 | 3  5  8 10
+   *       | 6  9 11 12
+   *
+   * First we iterate over the caps1 structures (top line) intersecting
+   * the structures diagonally down, then we iterate over the caps2
+   * structures.
+   */
+  for (i = 0; i < caps1->structs->len + caps2->structs->len - 1; i++) {
+    /* caps1 index goes from 0 to caps1->structs->len-1 */
+    j = MIN (i, caps1->structs->len - 1);
+    /* caps2 index stays 0 until i reaches caps1->structs->len, then it counts
+     * up from 1 to caps2->structs->len - 1 */
+    k = MAX (0, i - j);
+
+    /* now run the diagonal line, end condition is the left or bottom
+     * border */
+    while (k < caps2->structs->len) {
+      struct1 = gst_caps_get_structure (caps1, j);
+      struct2 = gst_caps_get_structure (caps2, k);
+
       istruct = gst_caps_structure_intersect (struct1, struct2);
 
       gst_caps_append_structure (dest, istruct);
+      /* move down left */
+      k++;
+      if (j == 0)
+        break;                  /* so we don't roll back to G_MAXUINT */
+      j--;
     }
   }
-
-  gst_caps_do_simplify (dest);
   return dest;
 }
 
@@ -861,8 +1083,8 @@ typedef struct
 SubtractionEntry;
 
 
-gboolean
-gst_caps_structure_subtract_field (GQuark field_id, GValue * value,
+static gboolean
+gst_caps_structure_subtract_field (GQuark field_id, const GValue * value,
     gpointer user_data)
 {
   SubtractionEntry *e = user_data;
@@ -927,7 +1149,7 @@ gst_caps_structure_subtract (GSList ** into, const GstStructure * minuend,
 GstCaps *
 gst_caps_subtract (const GstCaps * minuend, const GstCaps * subtrahend)
 {
-  int i, j;
+  guint i, j;
   GstStructure *min;
   GstStructure *sub;
   GstCaps *dest = NULL, *src;
@@ -953,7 +1175,7 @@ gst_caps_subtract (const GstCaps * minuend, const GstCaps * subtrahend)
   for (i = 0; i < subtrahend->structs->len; i++) {
     sub = gst_caps_get_structure (subtrahend, i);
     if (dest) {
-      gst_caps_free (src);
+      gst_caps_unref (src);
       src = dest;
     }
     dest = gst_caps_new_empty ();
@@ -977,12 +1199,12 @@ gst_caps_subtract (const GstCaps * minuend, const GstCaps * subtrahend)
       }
     }
     if (gst_caps_is_empty (dest)) {
-      gst_caps_free (src);
+      gst_caps_unref (src);
       return dest;
     }
   }
 
-  gst_caps_free (src);
+  gst_caps_unref (src);
   gst_caps_do_simplify (dest);
   return dest;
 }
@@ -1022,11 +1244,11 @@ typedef struct _NormalizeForeach
 NormalizeForeach;
 
 static gboolean
-gst_caps_normalize_foreach (GQuark field_id, GValue * value, gpointer ptr)
+gst_caps_normalize_foreach (GQuark field_id, const GValue * value, gpointer ptr)
 {
   NormalizeForeach *nf = (NormalizeForeach *) ptr;
   GValue val = { 0 };
-  int i;
+  guint i;
 
   if (G_VALUE_TYPE (value) == GST_TYPE_LIST) {
     for (i = 1; i < gst_value_list_get_size (value); i++) {
@@ -1061,7 +1283,7 @@ gst_caps_normalize (const GstCaps * caps)
 {
   NormalizeForeach nf;
   GstCaps *newcaps;
-  int i;
+  guint i;
 
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
 
@@ -1085,7 +1307,7 @@ gst_caps_compare_structures (gconstpointer one, gconstpointer two)
   const GstStructure *struct1 = *((const GstStructure **) one);
   const GstStructure *struct2 = *((const GstStructure **) two);
 
-  /* FIXME: this orders aphabetically, but ordering the quarks might be faster 
+  /* FIXME: this orders aphabetically, but ordering the quarks might be faster
      So what's the best way? */
   ret = strcmp (gst_structure_get_name (struct1),
       gst_structure_get_name (struct2));
@@ -1128,7 +1350,7 @@ typedef struct
 UnionField;
 
 static gboolean
-gst_caps_structure_figure_out_union (GQuark field_id, GValue * value,
+gst_caps_structure_figure_out_union (GQuark field_id, const GValue * value,
     gpointer user_data)
 {
   UnionField *u = user_data;
@@ -1216,13 +1438,23 @@ gst_caps_structure_simplify (GstStructure ** result,
   return FALSE;
 }
 
+static void
+gst_caps_switch_structures (GstCaps * caps, GstStructure * old,
+    GstStructure * new, gint i)
+{
+  gst_structure_set_parent_refcount (old, NULL);
+  gst_structure_free (old);
+  gst_structure_set_parent_refcount (new, &caps->refcount);
+  g_ptr_array_index (caps->structs, i) = new;
+}
+
 /**
  * gst_caps_do_simplify:
  * @caps: a #GstCaps to simplify
  *
- * Modifies the given @caps inplace into a representation that represents the 
- * same set of formats, but in a simpler form.  Component structures that are 
- * identical are merged.  Component structures that have values that can be 
+ * Modifies the given @caps inplace into a representation that represents the
+ * same set of formats, but in a simpler form.  Component structures that are
+ * identical are merged.  Component structures that have values that can be
  * merged are also merged.
  *
  * Returns: TRUE, if the caps could be simplified
@@ -1230,11 +1462,12 @@ gst_caps_structure_simplify (GstStructure ** result,
 gboolean
 gst_caps_do_simplify (GstCaps * caps)
 {
-  GstStructure *simplify, *compare, *result;
+  GstStructure *simplify, *compare, *result = NULL;
   gint i, j, start;
   gboolean changed = FALSE;
 
   g_return_val_if_fail (caps != NULL, FALSE);
+  g_return_val_if_fail (IS_WRITABLE (caps), FALSE);
 
   if (gst_caps_get_size (caps) < 2)
     return FALSE;
@@ -1256,15 +1489,8 @@ gst_caps_do_simplify (GstCaps * caps)
         break;
       }
       if (gst_caps_structure_simplify (&result, simplify, compare)) {
-#if 0
-        g_print ("%s  -  %s  =  %s\n",
-            gst_structure_to_string (simplify),
-            gst_structure_to_string (compare),
-            result ? gst_structure_to_string (result) : "---");
-#endif
         if (result) {
-          gst_structure_free (simplify);
-          g_ptr_array_index (caps->structs, i) = result;
+          gst_caps_switch_structures (caps, simplify, result, i);
           simplify = result;
         } else {
           gst_caps_remove_structure (caps, i);
@@ -1298,7 +1524,7 @@ gst_caps_save_thyself (const GstCaps * caps, xmlNodePtr parent)
 {
   char *s = gst_caps_to_string (caps);
 
-  xmlNewChild (parent, NULL, "caps", s);
+  xmlNewChild (parent, NULL, (xmlChar *) "caps", (xmlChar *) s);
   g_free (s);
   return parent;
 }
@@ -1314,8 +1540,8 @@ gst_caps_save_thyself (const GstCaps * caps, xmlNodePtr parent)
 GstCaps *
 gst_caps_load_thyself (xmlNodePtr parent)
 {
-  if (strcmp ("caps", parent->name) == 0) {
-    return gst_caps_from_string (xmlNodeGetContent (parent));
+  if (strcmp ("caps", (char *) parent->name) == 0) {
+    return gst_caps_from_string ((gchar *) xmlNodeGetContent (parent));
   }
 
   return NULL;
@@ -1329,21 +1555,29 @@ gst_caps_load_thyself (xmlNodePtr parent)
  * @caps: a pointer to #GstCaps
  * @newcaps: a #GstCaps to replace *caps
  *
- * Replaces *caps with @newcaps.  Frees the #GstCaps in the location
+ * Replaces *caps with @newcaps.  Unrefs the #GstCaps in the location
  * pointed to by @caps, if applicable, then modifies @caps to point to
- * @newcaps.
+ * @newcaps. An additional ref on @newcaps is taken.
  */
 void
 gst_caps_replace (GstCaps ** caps, GstCaps * newcaps)
 {
+  GstCaps *oldcaps;
+
 #if 0                           /* disable this, since too many plugins rely on undefined behavior */
 #ifdef USE_POISONING
   //if (newcaps) CAPS_POISON (newcaps);
 #endif
 #endif
-  if (*caps)
-    gst_caps_free (*caps);
+  oldcaps = *caps;
+
+  if (newcaps)
+    gst_caps_ref (newcaps);
+
   *caps = newcaps;
+
+  if (oldcaps)
+    gst_caps_unref (oldcaps);
 }
 
 /**
@@ -1358,10 +1592,8 @@ gst_caps_replace (GstCaps ** caps, GstCaps * newcaps)
 gchar *
 gst_caps_to_string (const GstCaps * caps)
 {
-  int i;
-  GstStructure *structure;
+  guint i;
   GString *s;
-  char *sstr;
 
   /* NOTE:  This function is potentially called by the debug system,
    * so any calls to gst_log() (and GST_DEBUG(), GST_LOG(), etc.)
@@ -1369,8 +1601,6 @@ gst_caps_to_string (const GstCaps * caps)
    * called by gst_caps_to_string.  In particular, calls should
    * not use the GST_PTR_FORMAT extension.  */
 
-  /* FIXME does this leak? */
-
   if (caps == NULL) {
     return g_strdup ("NULL");
   }
@@ -1380,16 +1610,16 @@ gst_caps_to_string (const GstCaps * caps)
   if (gst_caps_is_empty (caps)) {
     return g_strdup ("EMPTY");
   }
+
   s = g_string_new ("");
-  structure = gst_caps_get_structure (caps, 0);
-  sstr = gst_structure_to_string (structure);
-  g_string_append (s, sstr);
-  g_free (sstr);
+  for (i = 0; i < caps->structs->len; i++) {
+    GstStructure *structure;
+    char *sstr;
+
+    if (i > 0)
+      g_string_append (s, "; ");
 
-  for (i = 1; i < caps->structs->len; i++) {
     structure = gst_caps_get_structure (caps, i);
-
-    g_string_append (s, "; ");
     sstr = gst_structure_to_string (structure);
     g_string_append (s, sstr);
     g_free (sstr);
@@ -1456,7 +1686,7 @@ gst_caps_from_string (const gchar * string)
   if (gst_caps_from_string_inplace (caps, string)) {
     return caps;
   } else {
-    gst_caps_free (caps);
+    gst_caps_unref (caps);
     return NULL;
   }
 }
@@ -1475,140 +1705,11 @@ gst_caps_transform_to_string (const GValue * src_value, GValue * dest_value)
 }
 
 static GstCaps *
-gst_caps_copy_conditional (const GstCaps * src)
+gst_caps_copy_conditional (GstCaps * src)
 {
   if (src) {
-    return gst_caps_copy (src);
+    return gst_caps_ref (src);
   } else {
     return NULL;
   }
 }
-
-/* fixate utility functions */
-
-/**
- * gst_caps_structure_fixate_field_nearest_int:
- * @structure: a #GstStructure
- * @field_name: a field in @structure
- * @target: the target value of the fixation
- *
- * Fixates a #GstStructure by changing the given field to the nearest
- * integer to @target that is a subset of the existing field.
- *
- * Returns: TRUE if the structure could be fixated
- */
-gboolean
-gst_caps_structure_fixate_field_nearest_int (GstStructure * structure,
-    const char *field_name, int target)
-{
-  const GValue *value;
-
-  g_return_val_if_fail (gst_structure_has_field (structure, field_name), FALSE);
-
-  value = gst_structure_get_value (structure, field_name);
-
-  if (G_VALUE_TYPE (value) == G_TYPE_INT) {
-    /* already fixed */
-    return FALSE;
-  } else if (G_VALUE_TYPE (value) == GST_TYPE_INT_RANGE) {
-    int x;
-
-    x = gst_value_get_int_range_min (value);
-    if (target < x)
-      target = x;
-    x = gst_value_get_int_range_max (value);
-    if (target > x)
-      target = x;
-    gst_structure_set (structure, field_name, G_TYPE_INT, target, NULL);
-    return TRUE;
-  } else if (G_VALUE_TYPE (value) == GST_TYPE_LIST) {
-    const GValue *list_value;
-    int i, n;
-    int best = 0;
-    int best_index = -1;
-
-    n = gst_value_list_get_size (value);
-    for (i = 0; i < n; i++) {
-      list_value = gst_value_list_get_value (value, i);
-      if (G_VALUE_TYPE (list_value) == G_TYPE_INT) {
-        int x = g_value_get_int (list_value);
-
-        if (best_index == -1 || (ABS (target - x) < ABS (target - best))) {
-          best_index = i;
-          best = x;
-        }
-      }
-    }
-    if (best_index != -1) {
-      gst_structure_set (structure, field_name, G_TYPE_INT, best, NULL);
-      return TRUE;
-    }
-    return FALSE;
-  }
-
-  return FALSE;
-}
-
-/**
- * gst_caps_structure_fixate_field_nearest_double:
- * @structure: a #GstStructure
- * @field_name: a field in @structure
- * @target: the target value of the fixation
- *
- * Fixates a #GstStructure by changing the given field to the nearest
- * double to @target that is a subset of the existing field.
- *
- * Returns: TRUE if the structure could be fixated
- */
-gboolean
-gst_caps_structure_fixate_field_nearest_double (GstStructure * structure,
-    const char *field_name, double target)
-{
-  const GValue *value;
-
-  g_return_val_if_fail (gst_structure_has_field (structure, field_name), FALSE);
-
-  value = gst_structure_get_value (structure, field_name);
-
-  if (G_VALUE_TYPE (value) == G_TYPE_DOUBLE) {
-    /* already fixed */
-    return FALSE;
-  } else if (G_VALUE_TYPE (value) == GST_TYPE_DOUBLE_RANGE) {
-    double x;
-
-    x = gst_value_get_double_range_min (value);
-    if (target < x)
-      target = x;
-    x = gst_value_get_double_range_max (value);
-    if (target > x)
-      target = x;
-    gst_structure_set (structure, field_name, G_TYPE_DOUBLE, target, NULL);
-    return TRUE;
-  } else if (G_VALUE_TYPE (value) == GST_TYPE_LIST) {
-    const GValue *list_value;
-    int i, n;
-    double best = 0;
-    int best_index = -1;
-
-    n = gst_value_list_get_size (value);
-    for (i = 0; i < n; i++) {
-      list_value = gst_value_list_get_value (value, i);
-      if (G_VALUE_TYPE (list_value) == G_TYPE_DOUBLE) {
-        double x = g_value_get_double (list_value);
-
-        if (best_index == -1 || (ABS (target - x) < ABS (target - best))) {
-          best_index = i;
-          best = x;
-        }
-      }
-    }
-    if (best_index != -1) {
-      gst_structure_set (structure, field_name, G_TYPE_DOUBLE, best, NULL);
-      return TRUE;
-    }
-    return FALSE;
-  }
-
-  return FALSE;
-
-}
