@@ -20,13 +20,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstindex
+ * @short_description: Generate indexes on objects
+ * @see_also: #GstIndexFactory
+ *
+ * GstIndex is used to generate a stream index of one or more elements
+ * in a pipeline.
+ */
+
 #include "gst_private.h"
 
 #include "gstinfo.h"
-#include "gstregistrypool.h"
-#include "gstpad.h"
 #include "gstindex.h"
+#include "gstindexfactory.h"
 #include "gstmarshal.h"
+#include "gstregistry.h"
+/* for constructing an entry name */
+#include "gstelement.h"
+#include "gstpad.h"
 
 /* Index signals and args */
 enum
@@ -147,6 +159,13 @@ gst_index_class_init (GstIndexClass * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_OBJECT);
 
+  /**
+   * GstIndex::entry-added
+   * @gstindex: the object which received the signal.
+   * @arg1: The entry added to the index.
+   *
+   * Is emitted when a new entry is added to the index.
+   */
   gst_index_signals[ENTRY_ADDED] =
       g_signal_new ("entry-added", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstIndexClass, entry_added), NULL, NULL,
@@ -175,8 +194,8 @@ gst_index_init (GstIndex * index)
   index->resolver = resolvers[index->method].resolver;
   index->resolver_user_data = resolvers[index->method].user_data;
 
-  GST_FLAG_SET (index, GST_INDEX_WRITABLE);
-  GST_FLAG_SET (index, GST_INDEX_READABLE);
+  GST_OBJECT_FLAG_SET (index, GST_INDEX_WRITABLE);
+  GST_OBJECT_FLAG_SET (index, GST_INDEX_READABLE);
 
   GST_DEBUG ("created new index");
 }
@@ -383,8 +402,31 @@ gst_index_set_filter (GstIndex * index,
 {
   g_return_if_fail (GST_IS_INDEX (index));
 
+  gst_index_set_filter_full (index, filter, user_data, NULL);
+}
+
+/**
+ * gst_index_set_filter_full:
+ * @index: the index to register the filter on
+ * @filter: the filter to register
+ * @user_data: data passed to the filter function
+ * @user_data_destroy: function to call when @user_data is unset
+ *
+ * Lets the app register a custom filter function so that
+ * it can select what entries should be stored in the index.
+ */
+void
+gst_index_set_filter_full (GstIndex * index,
+    GstIndexFilter filter, gpointer user_data, GDestroyNotify user_data_destroy)
+{
+  g_return_if_fail (GST_IS_INDEX (index));
+
+  if (index->filter_user_data && index->filter_user_data_destroy)
+    index->filter_user_data_destroy (index->filter_user_data);
+
   index->filter = filter;
   index->filter_user_data = user_data;
+  index->filter_user_data_destroy = user_data_destroy;
 }
 
 /**
@@ -515,7 +557,8 @@ gst_index_gtype_resolver (GstIndex * index, GstObject * writer,
     gchar ** writer_string, gpointer data)
 {
   if (GST_IS_PAD (writer)) {
-    GstElement *element = gst_pad_get_parent (GST_PAD (writer));
+    GstElement *element =
+        (GstElement *) gst_object_get_parent (GST_OBJECT (writer));
 
     *writer_string = g_strdup_printf ("%s.%s",
         g_type_name (G_OBJECT_TYPE (element)), gst_object_get_name (writer));
@@ -535,9 +578,9 @@ gst_index_gtype_resolver (GstIndex * index, GstObject * writer,
  *
  * Before entries can be added to the index, a writer
  * should obtain a unique id. The methods to add new entries
- * to the index require this id as an argument. 
+ * to the index require this id as an argument.
  *
- * The application can implement a custom function to map the writer object 
+ * The application can implement a custom function to map the writer object
  * to a string. That string will be used to register or look up an id
  * in the index.
  *
@@ -662,7 +705,7 @@ gst_index_add_associationv (GstIndex * index, gint id, GstAssocFlags flags,
  * @id: the id of the index writer
  * @flags: optinal flags for this entry
  * @format: the format of the value
- * @value: the value 
+ * @value: the value
  * @...: other format/value pairs or 0 to end the list
  *
  * Associate given format/value pairs with each other.
@@ -725,7 +768,9 @@ gst_index_add_association (GstIndex * index, gint id, GstAssocFlags flags,
  * @object: a pointer to the object to add
  *
  * Add the given object to the index with the given key.
- * 
+ *
+ * This function is not yet implemented.
+ *
  * Returns: a pointer to the newly added entry in the index.
  */
 GstIndexEntry *
@@ -741,7 +786,11 @@ gst_index_add_object (GstIndex * index, gint id, gchar * key,
 static gint
 gst_index_compare_func (gconstpointer a, gconstpointer b, gpointer user_data)
 {
-  return (gint) a - (gint) b;
+  if (a < b)
+    return -1;
+  if (a > b)
+    return 1;
+  return 0;
 }
 
 /**
@@ -818,7 +867,7 @@ gst_index_get_assoc_entry_full (GstIndex * index, gint id,
  *
  * Gets alternative formats associated with the indexentry.
  *
- * Returns: TRUE if there was a value associated with the given 
+ * Returns: TRUE if there was a value associated with the given
  * format.
  */
 gboolean
@@ -838,172 +887,3 @@ gst_index_entry_assoc_map (GstIndexEntry * entry,
   }
   return FALSE;
 }
-
-
-static void gst_index_factory_class_init (GstIndexFactoryClass * klass);
-static void gst_index_factory_init (GstIndexFactory * factory);
-
-static GstPluginFeatureClass *factory_parent_class = NULL;
-
-/* static guint gst_index_factory_signals[LAST_SIGNAL] = { 0 }; */
-
-GType
-gst_index_factory_get_type (void)
-{
-  static GType indexfactory_type = 0;
-
-  if (!indexfactory_type) {
-    static const GTypeInfo indexfactory_info = {
-      sizeof (GstIndexFactoryClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_index_factory_class_init,
-      NULL,
-      NULL,
-      sizeof (GstIndexFactory),
-      0,
-      (GInstanceInitFunc) gst_index_factory_init,
-      NULL
-    };
-
-    indexfactory_type = g_type_register_static (GST_TYPE_PLUGIN_FEATURE,
-        "GstIndexFactory", &indexfactory_info, 0);
-  }
-  return indexfactory_type;
-}
-
-static void
-gst_index_factory_class_init (GstIndexFactoryClass * klass)
-{
-  GObjectClass *gobject_class;
-  GstObjectClass *gstobject_class;
-  GstPluginFeatureClass *gstpluginfeature_class;
-
-  gobject_class = (GObjectClass *) klass;
-  gstobject_class = (GstObjectClass *) klass;
-  gstpluginfeature_class = (GstPluginFeatureClass *) klass;
-
-  factory_parent_class = g_type_class_ref (GST_TYPE_PLUGIN_FEATURE);
-}
-
-static void
-gst_index_factory_init (GstIndexFactory * factory)
-{
-}
-
-/**
- * gst_index_factory_new:
- * @name: name of indexfactory to create
- * @longdesc: long description of indexfactory to create
- * @type: the GType of the GstIndex element of this factory
- *
- * Create a new indexfactory with the given parameters
- *
- * Returns: a new #GstIndexFactory.
- */
-GstIndexFactory *
-gst_index_factory_new (const gchar * name, const gchar * longdesc, GType type)
-{
-  GstIndexFactory *factory;
-
-  g_return_val_if_fail (name != NULL, NULL);
-  factory = gst_index_factory_find (name);
-  if (!factory) {
-    factory = GST_INDEX_FACTORY (g_object_new (GST_TYPE_INDEX_FACTORY, NULL));
-  }
-
-  GST_PLUGIN_FEATURE_NAME (factory) = g_strdup (name);
-  if (factory->longdesc)
-    g_free (factory->longdesc);
-  factory->longdesc = g_strdup (longdesc);
-  factory->type = type;
-
-  return factory;
-}
-
-/**
- * gst_index_factory_destroy:
- * @factory: factory to destroy
- *
- * Removes the index from the global list.
- */
-void
-gst_index_factory_destroy (GstIndexFactory * factory)
-{
-  g_return_if_fail (factory != NULL);
-
-  /* we don't free the struct bacause someone might  have a handle to it.. */
-}
-
-/**
- * gst_index_factory_find:
- * @name: name of indexfactory to find
- *
- * Search for an indexfactory of the given name.
- *
- * Returns: #GstIndexFactory if found, NULL otherwise
- */
-GstIndexFactory *
-gst_index_factory_find (const gchar * name)
-{
-  GstPluginFeature *feature;
-
-  g_return_val_if_fail (name != NULL, NULL);
-
-  GST_DEBUG ("gstindex: find \"%s\"", name);
-
-  feature = gst_registry_pool_find_feature (name, GST_TYPE_INDEX_FACTORY);
-  if (feature)
-    return GST_INDEX_FACTORY (feature);
-
-  return NULL;
-}
-
-/**
- * gst_index_factory_create:
- * @factory: the factory used to create the instance
- *
- * Create a new #GstIndex instance from the 
- * given indexfactory.
- *
- * Returns: A new #GstIndex instance.
- */
-GstIndex *
-gst_index_factory_create (GstIndexFactory * factory)
-{
-  GstIndex *new = NULL;
-
-  g_return_val_if_fail (factory != NULL, NULL);
-
-  if (gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
-    g_return_val_if_fail (factory->type != 0, NULL);
-
-    new = GST_INDEX (g_object_new (factory->type, NULL));
-  }
-
-  return new;
-}
-
-/**
- * gst_index_factory_make:
- * @name: the name of the factory used to create the instance
- *
- * Create a new #GstIndex instance from the 
- * indexfactory with the given name.
- *
- * Returns: A new #GstIndex instance.
- */
-GstIndex *
-gst_index_factory_make (const gchar * name)
-{
-  GstIndexFactory *factory;
-
-  g_return_val_if_fail (name != NULL, NULL);
-
-  factory = gst_index_factory_find (name);
-
-  if (factory == NULL)
-    return NULL;
-
-  return gst_index_factory_create (factory);
-}
