@@ -20,6 +20,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gsturihandler
+ * @short_description: Interface to ease URI handling in plugins.
+ *
+ * The URIHandler is an interface that is implemented by Source and Sink 
+ * #GstElement to simplify then handling of URI.
+ *
+ * An application can use the following functions to quickly get an element
+ * that handles the given URI for reading or writing (#gst_element_make_from_uri).
+ *
+ * Source and Sink plugins should implement when possible this interface.
+ *
+ * Last reviewed on 2005-11-09 (0.9.4)
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -27,8 +42,8 @@
 #include "gst_private.h"
 #include "gsturi.h"
 #include "gstinfo.h"
-#include "gstregistrypool.h"
 #include "gstmarshal.h"
+#include "gstregistry.h"
 
 #include <string.h>
 
@@ -70,6 +85,15 @@ gst_uri_handler_base_init (gpointer g_class)
   static gboolean initialized = FALSE;
 
   if (!initialized) {
+
+    /**
+     * GstURIHandler::new-uri:
+     * @handler: The #GstURIHandler which emitted the signal
+     * @uri: The new URI, or NULL if the URI was removed
+     *
+     * The URI of the given @handler has changed.
+     */
+
     g_signal_new ("new-uri", GST_TYPE_URI_HANDLER, G_SIGNAL_RUN_LAST,
         G_STRUCT_OFFSET (GstURIHandlerInterface, new_uri), NULL, NULL,
         gst_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
@@ -175,13 +199,13 @@ escape_string_internal (const gchar * string, UnsafeCharacterSet mask)
  * escape_string:
  * @string: string to be escaped
  *
- * Escapes @string, replacing any and all special characters 
+ * Escapes @string, replacing any and all special characters
  * with equivalent escape sequences.
  *
  * Return value: a newly allocated string equivalent to @string
  * but with all special characters escaped
  **/
-gchar *
+static gchar *
 escape_string (const gchar * string)
 {
   return escape_string_internal (string, UNSAFE_ALL);
@@ -221,11 +245,11 @@ unescape_character (const char *scanner)
  * considered "illegal", '\0' is automatically in this list.
  *
  * Decodes escaped characters (i.e. PERCENTxx sequences) in @escaped_string.
- * Characters are encoded in PERCENTxy form, where xy is the ASCII hex code 
+ * Characters are encoded in PERCENTxy form, where xy is the ASCII hex code
  * for character 16x+y.
- * 
- * Return value: a newly allocated string with the unescaped equivalents, 
- * or %NULL if @escaped_string contained one of the characters 
+ *
+ * Return value: a newly allocated string with the unescaped equivalents,
+ * or %NULL if @escaped_string contained one of the characters
  * in @illegal_characters.
  **/
 static char *
@@ -260,7 +284,7 @@ unescape_string (const gchar * escaped_string, const gchar * illegal_characters)
   }
 
   *out = '\0';
-  g_assert (out - result <= strlen (escaped_string));
+  g_assert ((size_t) (out - result) <= strlen (escaped_string));
   return result;
 
 }
@@ -285,12 +309,12 @@ gst_uri_protocol_check_internal (const gchar * uri, gchar ** endptr)
 
 /**
  * gst_uri_protocol_is_valid:
- * @protocol: string to check
+ * @protocol: A string
  *
  * Tests if the given string is a valid protocol identifier. Protocols
  * must consist of alphanumeric characters and not start with a number.
  *
- * Returns: TRUE if the string is a valid protocol identifier
+ * Returns: TRUE if the string is a valid protocol identifier, FALSE otherwise.
  */
 gboolean
 gst_uri_protocol_is_valid (const gchar * protocol)
@@ -306,10 +330,10 @@ gst_uri_protocol_is_valid (const gchar * protocol)
 
 /**
  * gst_uri_is_valid:
- * @uri: string to check
+ * @uri: A URI string
  *
  * Tests if the given string is a valid URI identifier. URIs start with a valid
- * protocol followed by "://" and a string identifying the location.
+ * protocol followed by "://" and maybe a string identifying the location.
  *
  * Returns: TRUE if the string is a valid URI
  */
@@ -327,7 +351,7 @@ gst_uri_is_valid (const gchar * uri)
 
 /**
  * gst_uri_get_protocol:
- * @uri: URI to get protocol from
+ * @uri: A URI string
  *
  * Extracts the protocol out of a given valid URI. The returned string must be
  * freed using g_free().
@@ -349,13 +373,13 @@ gst_uri_get_protocol (const gchar * uri)
 
 /**
  * gst_uri_get_location:
- * @uri: URI to get the location from
+ * @uri: A URI string
  *
  * Extracts the location out of a given valid URI. So the protocol and "://"
- * are stripped from the URI. The returned string must be freed using 
+ * are stripped from the URI. The returned string must be freed using
  * g_free().
  *
- * Returns: The location for this URI.
+ * Returns: The location for this URI. Returns NULL if the URI isn't valid.
  */
 gchar *
 gst_uri_get_location (const gchar * uri)
@@ -378,12 +402,13 @@ gst_uri_get_location (const gchar * uri)
 
 /**
  * gst_uri_construct:
- * @protocol: protocol for URI
- * @location: location for URI
+ * @protocol: Protocol for URI
+ * @location: Location for URI
  *
  * Constructs a URI for a given valid protocol and location.
  *
- * Returns: a new string for this URI
+ * Returns: a new string for this URI. Returns NULL if the given URI protocol
+ * is not valid, or the given location is NULL.
  */
 gchar *
 gst_uri_construct (const gchar * protocol, const gchar * location)
@@ -445,12 +470,12 @@ sort_by_rank (gconstpointer a, gconstpointer b)
 
 /**
  * gst_element_make_from_uri:
- * @type: wether to create a source or a sink
- * @uri: URI to create element for
- * @elementname: optional name of created element
+ * @type: Wether to create a source or a sink
+ * @uri: URI to create an element for
+ * @elementname: Name of created element, can be NULL.
+ *
+ * Creates an element for handling the given URI.
  *
- * Creates an element for handling the given URI. 
- * 
  * Returns: a new element or NULL if none could be created
  */
 GstElement *
@@ -466,8 +491,8 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
 
   entry.type = type;
   entry.protocol = gst_uri_get_protocol (uri);
-  possibilities =
-      gst_registry_pool_feature_filter (search_by_entry, FALSE, &entry);
+  possibilities = gst_registry_feature_filter (gst_registry_get_default (),
+      search_by_entry, FALSE, &entry);
   g_free (entry.protocol);
 
   if (!possibilities) {
@@ -485,7 +510,7 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
 
       if (gst_uri_handler_set_uri (handler, uri))
         break;
-      gst_object_unref (GST_OBJECT (ret));
+      gst_object_unref (ret);
       ret = NULL;
     }
     walk = walk->next;
@@ -499,11 +524,12 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
 
 /**
  * gst_uri_handler_get_uri_type:
- * @handler: Handler to query type of
+ * @handler: A #GstURIHandler.
  *
- * Gets the type of a URI handler
+ * Gets the type of the given URI handler
  *
- * Returns: the type of the URI handler
+ * Returns: the #GstURIType of the URI handler.
+ * Returns #GST_URI_UNKNOWN if the @handler isn't implemented correctly.
  */
 guint
 gst_uri_handler_get_uri_type (GstURIHandler * handler)
@@ -524,12 +550,14 @@ gst_uri_handler_get_uri_type (GstURIHandler * handler)
 
 /**
  * gst_uri_handler_get_protocols:
- * @handler: Handler to get protocols for
+ * @handler: A #GstURIHandler.
  *
- * Gets the list of supported protocols for this handler. This list may not be
+ * Gets the list of protocols supported by @handler. This list may not be
  * modified.
  *
- * Returns: the supported protocols
+ * Returns: the supported protocols.
+ * Returns NULL if the @handler isn't implemented properly, or the @handler
+ * doesn't support any protocols.
  */
 gchar **
 gst_uri_handler_get_protocols (GstURIHandler * handler)
@@ -550,11 +578,12 @@ gst_uri_handler_get_protocols (GstURIHandler * handler)
 
 /**
  * gst_uri_handler_get_uri:
- * @handler: handler to query URI of
+ * @handler: A #GstURIHandler
  *
- * Gets the currently handled URI of the handler or NULL, if none is set.
+ * Gets the currently handled URI.
  *
- * Returns: the URI
+ * Returns: the URI currently handler by the @handler.
+ * Returns NULL if there are no URI currently handled.
  */
 G_CONST_RETURN gchar *
 gst_uri_handler_get_uri (GstURIHandler * handler)
@@ -576,12 +605,12 @@ gst_uri_handler_get_uri (GstURIHandler * handler)
 
 /**
  * gst_uri_handler_set_uri:
- * @handler: handler to set URI of
+ * @handler: A #GstURIHandler
  * @uri: URI to set
  *
- * Tries to set the URI of the given handler and returns TRUE if it succeeded.
+ * Tries to set the URI of the given handler.
  *
- * Returns: TRUE, if the URI was set successfully
+ * Returns: TRUE if the URI was set successfully, else FALSE.
  */
 gboolean
 gst_uri_handler_set_uri (GstURIHandler * handler, const gchar * uri)
@@ -599,10 +628,10 @@ gst_uri_handler_set_uri (GstURIHandler * handler, const gchar * uri)
 
 /**
  * gst_uri_handler_new_uri:
- * @handler: handler with a new URI
+ * @handler: A #GstURIHandler
  * @uri: new URI or NULL if it was unset
  *
- * Emits the new-uri event for a given handler, when that handler has a new URI.
+ * Emits the new-uri signal for a given handler, when that handler has a new URI.
  * This function should only be called by URI handlers themselves.
  */
 void
